#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "circuits.h"


static int line;
const char *fname;

const char *GateNames[7] = {
	"Invalid", 
	"AND", 
	"OR", 
	"NAND", 
	"NOR", 
	"NOT", 
	"XOR"
};

#define PRINTF(s,p) printf( "%s, line %d: ", fname, line ); printf( s, p )
#define WARNING(s,p) PRINTF(s,p)



int GateStr2Int(const char *name) {
	if( strcmp( "AND", name ) == 0 )
		return G_AND;
	if( strcmp( "OR", name ) == 0 )
		return G_OR;
	if( strcmp( "XOR", name ) == 0 )
		return G_XOR;
	if( strcmp( "NOT", name ) == 0 )
		return G_NOT;
	if( strcmp( "NAND", name ) == 0 )
		return G_NAND;
	if( strcmp( "NOR", name ) == 0 )
		return G_NOR;
	PRINTF( "Gate name not recognized (%s)\nSupported gate names are AND, NAND, OR, NOR, XOR and NOT\n", name );
	exit( 0 );
}


void ZeroMem(int *p,int length) {
	int i;
	for(i=0; i<length; i++)
		*p ++ = 0;
}


void InputGate(int g[C_GATES][4],int layer,int gate, int input1, int input2, int output) {
	int i;

	for(i=0; i<C_GATES; i++) {
		if( !g[i][0] )
			break;
	}
	if( i == C_GATES ) {
		WARNING( "Gates for layer %d are already filled\nGate will be ignored\n", layer );
		return;
	}

	g[i][0] = gate;
	g[i][1] = input1;
	if( gate != G_NOT) g[i][2] = input2;
	else g[i][2] = -1;	//	invalid input, should not be used
	g[i][3] = output;
}

void PrintGate(int *settings) {
	if( !settings[0] )
		return;
	if( settings[0] != G_NOT )
		printf( "%s, Input1: %d, Input2: %d, Output: %d\n",
			GateNames[settings[0]],
			settings[1],
			settings[2],
			settings[3]
			);
	else
		printf( "%s, Input1: %d, Input2: Not applicable, Output: %d\n",
			GateNames[settings[0]],
			settings[1],
			settings[3]
			);
}

void LoadCircuit(const char *filename) {
	FILE *fp;
	char buf[1001];

	fname = filename;
	fp = fopen(filename,"r");
	if( !fp ) {
		printf( "Can not open %s: ", filename );
		perror( "" );
		exit( 0 );
	}

	ZeroMem((int *) gates,B_COLUMNS*C_GATES*4);

	line = 1;
	while( fgets(buf,1000,fp) ) {
		char *name;
		int layer = -1, input1 = -1, input2 = -1, output = -1;
		int gate, length;

		name = buf;
		length = strlen(buf);
		while( *name ++ != ',' && length > 0 )
			length --;
		name[-1] = 0;
		name = buf;
		if( length == 0 ) {
			PRINTF( "Invalid format of line (%s)\n", buf );
			exit( 0 );
		}
		sscanf( buf+strlen(name)+1, "%X,%d,%d,%d",
			&layer, &input1, &input2, &output );
		layer -= 0xC1;

		gate = GateStr2Int(name);
		if( gate == G_NOT ) {
			output = input2;
		}
		if( layer < 0 || layer > 2 ) {
			PRINTF( "Layer for gate invalid (%2.2X)\nGate must be plugged to layers 1-3\n", layer + 0xC1);
			exit( 0 );
		}
		if( input1 < 0 || input1 > 9 ) {
			PRINTF( "Input1 pin for gate invalid (%d)\nSupported input pins are 1-10\n", input1 );
			exit( 0 );
		}
		if( gate != G_NOT && (input2 < 0 || input2 > 9) ) {
			PRINTF( "Input2 pin for gate invalid (%d)\nSupported input pins are 1-10\n", input2 );
			exit( 0 );
		}
		if( output < 0 || output > 9 ) {
			PRINTF( "Output pin for gate invalid (%d)\nSupported output pins are 1-10\n", output );
			exit( 0 );
		}

		InputGate(gates[layer],
			layer, gate,input1,input2,output);


		line ++;
	}

	{	//	Print gates
		int i, j;
		printf( "\n\n" );
		for(i=0; i<B_COLUMNS; i++) {
			printf( "Column #%d Gates:\n", i+1 );
			for(j=0; j<C_GATES; j++) {
				PrintGate( gates[i][j] );
			}
		}
	}

	fclose( fp );
}
